@@ -15,13 +15,14 @@
 #define TCONTEXT "tcontext="
 #define TCLASS "tclass="
 
-void usage(char *progname, int rc) 
+void usage(char *progname, int rc)
 {
- 	fprintf(stderr, "usage:  %s [-p policy] < /var/log/audit/audit.log\n", progname);
+	fprintf(stderr, "usage:  %s [-p policy] < /var/log/audit/audit.log\n",
+		progname);
 	exit(rc);
 }
 
-int main(int argc, char **argv) 
+int main(int argc, char **argv)
 {
 	char path[PATH_MAX];
 	char *buffer = NULL, *bufcopy = NULL;
@@ -62,25 +63,30 @@ int main(int argc, char **argv)
 
 	if (!set_path) {
 		if (!is_selinux_enabled()) {
-			fprintf(stderr, "%s:  Must specify -p policy on non-SELinux systems\n", argv[0]);
+			fprintf(stderr,
+				"%s:  Must specify -p policy on non-SELinux systems\n",
+				argv[0]);
 			exit(1);
 		}
 		vers = security_policyvers();
 		if (vers < 0) {
-			fprintf(stderr, "%s:  Could not get policy version:  %s\n", argv[0], strerror(errno));
+			fprintf(stderr,
+				"%s:  Could not get policy version:  %s\n",
+				argv[0], strerror(errno));
 			exit(1);
 		}
-		snprintf(path, PATH_MAX, "%s.%d", 
+		snprintf(path, PATH_MAX, "%s.%d",
 			 selinux_binary_policy_path(), vers);
 		fp = fopen(path, "r");
 		while (!fp && errno == ENOENT && --vers) {
-			snprintf(path, PATH_MAX, "%s.%d", 
+			snprintf(path, PATH_MAX, "%s.%d",
 				 selinux_binary_policy_path(), vers);
 			fp = fopen(path, "r");
 		}
 		if (!fp) {
-			snprintf(path, PATH_MAX, "%s.%d", 
-				 selinux_binary_policy_path(), security_policyvers());
+			snprintf(path, PATH_MAX, "%s.%d",
+				 selinux_binary_policy_path(),
+				 security_policyvers());
 			fprintf(stderr, "%s:  unable to open %s:  %s\n",
 				argv[0], path, strerror(errno));
 			exit(1);
@@ -114,17 +120,16 @@ int main(int argc, char **argv)
 		   and then apply audit2why -p to the resulting file. 
 		   Errors are non-fatal as such settings are optional. */
 		sepol_debug(0);
-		(void)sepol_genbools_policydb(&policydb, selinux_booleans_path());
+		(void)sepol_genbools_policydb(&policydb,
+					      selinux_booleans_path());
 		(void)sepol_genusers_policydb(&policydb, selinux_users_path());
 	}
 
-
 	/* Initialize the sidtab for subsequent use by sepol_context_to_sid
 	   and sepol_compute_av_reason. */
 	rc = sepol_sidtab_init(&sidtab);
 	if (rc < 0) {
-		fprintf(stderr, "%s:  unable to init sidtab\n",
-			argv[0]);
+		fprintf(stderr, "%s:  unable to init sidtab\n", argv[0]);
 		exit(1);
 	}
 	sepol_set_sidtab(&sidtab);
@@ -133,15 +138,15 @@ int main(int argc, char **argv)
 	while (getline(&buffer, &len, stdin) > 0) {
 		size_t len2 = strlen(buffer);
 
-		if (buffer[len2-1] == '\n') 
-			buffer[len2-1] = 0;
+		if (buffer[len2 - 1] == '\n')
+			buffer[len2 - 1] = 0;
 		lineno++;
 
 		p = buffer;
 		while (*p && strncmp(p, AVCPREFIX, sizeof(AVCPREFIX) - 1))
 			p++;
 		if (!(*p))
-			continue; /* not an avc denial */
+			continue;	/* not an avc denial */
 
 		p += sizeof(AVCPREFIX) - 1;
 
@@ -164,19 +169,22 @@ int main(int argc, char **argv)
 		/* Remember where the permission list begins,
 		   and terminate the list. */
 		permstr = p;
-		while (*p && *p != '}') 
+		while (*p && *p != '}')
 			p++;
 		if (!(*p)) {
-			fprintf(stderr, "Missing closing bracket on line %u, skipping...\n", lineno);
+			fprintf(stderr,
+				"Missing closing bracket on line %u, skipping...\n",
+				lineno);
 			continue;
 		}
 		*p++ = 0;
 
 		/* Get scontext and convert to SID. */
-		while (*p && strncmp(p, SCONTEXT, sizeof(SCONTEXT)-1))
+		while (*p && strncmp(p, SCONTEXT, sizeof(SCONTEXT) - 1))
 			p++;
 		if (!(*p)) {
-			fprintf(stderr, "Missing %s on line %u, skipping...\n", SCONTEXT, lineno);
+			fprintf(stderr, "Missing %s on line %u, skipping...\n",
+				SCONTEXT, lineno);
 			continue;
 		}
 		p += sizeof(SCONTEXT) - 1;
@@ -185,17 +193,20 @@ int main(int argc, char **argv)
 			p++;
 		if (*p)
 			*p++ = 0;
-		rc = sepol_context_to_sid(scon, strlen(scon)+1, &ssid);
+		rc = sepol_context_to_sid(scon, strlen(scon) + 1, &ssid);
 		if (rc < 0) {
-			fprintf(stderr, "Invalid %s%s on line %u, skipping...\n", SCONTEXT, scon, lineno);
+			fprintf(stderr,
+				"Invalid %s%s on line %u, skipping...\n",
+				SCONTEXT, scon, lineno);
 			continue;
 		}
 
 		/* Get tcontext and convert to SID. */
-		while (*p && strncmp(p, TCONTEXT, sizeof(TCONTEXT)-1))
+		while (*p && strncmp(p, TCONTEXT, sizeof(TCONTEXT) - 1))
 			p++;
 		if (!(*p)) {
-			fprintf(stderr, "Missing %s on line %u, skipping...\n", TCONTEXT, lineno);
+			fprintf(stderr, "Missing %s on line %u, skipping...\n",
+				TCONTEXT, lineno);
 			continue;
 		}
 		p += sizeof(TCONTEXT) - 1;
@@ -204,9 +215,11 @@ int main(int argc, char **argv)
 			p++;
 		if (*p)
 			*p++ = 0;
-		rc = sepol_context_to_sid(tcon, strlen(tcon)+1, &tsid);
+		rc = sepol_context_to_sid(tcon, strlen(tcon) + 1, &tsid);
 		if (rc < 0) {
-			fprintf(stderr, "Invalid %s%s on line %u, skipping...\n", TCONTEXT, tcon, lineno);
+			fprintf(stderr,
+				"Invalid %s%s on line %u, skipping...\n",
+				TCONTEXT, tcon, lineno);
 			continue;
 		}
 
@@ -214,7 +227,8 @@ int main(int argc, char **argv)
 		while (*p && strncmp(p, TCLASS, sizeof(TCLASS) - 1))
 			p++;
 		if (!(*p)) {
-			fprintf(stderr, "Missing %s on line %u, skipping...\n", TCLASS, lineno);
+			fprintf(stderr, "Missing %s on line %u, skipping...\n",
+				TCLASS, lineno);
 			continue;
 		}
 		p += sizeof(TCLASS) - 1;
@@ -225,7 +239,9 @@ int main(int argc, char **argv)
 			*p = 0;
 		tclass = string_to_security_class(tclassstr);
 		if (!tclass) {
-			fprintf(stderr, "Invalid %s%s on line %u, skipping...\n", TCLASS, tclassstr, lineno);
+			fprintf(stderr,
+				"Invalid %s%s on line %u, skipping...\n",
+				TCLASS, tclassstr, lineno);
 			continue;
 		}
 
@@ -239,7 +255,9 @@ int main(int argc, char **argv)
 				*p++ = 0;
 			perm = string_to_av_perm(tclass, permstr);
 			if (!perm) {
-				fprintf(stderr, "Invalid permission %s on line %u, skipping...\n", permstr, lineno);
+				fprintf(stderr,
+					"Invalid permission %s on line %u, skipping...\n",
+					permstr, lineno);
 				continue;
 			}
 			av |= perm;
@@ -250,28 +268,36 @@ int main(int argc, char **argv)
 		rc = sepol_compute_av_reason(ssid, tsid, tclass, av, &avd,
 					     &reason);
 		if (rc < 0) {
-			fprintf(stderr, "Error during access vector computation on line %u, skipping...\n", lineno);
+			fprintf(stderr,
+				"Error during access vector computation on line %u, skipping...\n",
+				lineno);
 			continue;
 		}
 
 		printf("%s\n\tWas caused by:\n", bufcopy);
 
 		if (!reason) {
-			printf("\t\tUnknown - would be allowed by %s policy\n", set_path ? "specified" : "active");
-			printf("\t\tPossible mismatch between this policy and the one under which the audit message was generated.\n");
-			printf("\t\tPossible mismatch between current in-memory boolean settings vs. permanent ones.\n");
+			printf("\t\tUnknown - would be allowed by %s policy\n",
+			       set_path ? "specified" : "active");
+			printf
+			    ("\t\tPossible mismatch between this policy and the one under which the audit message was generated.\n");
+			printf
+			    ("\t\tPossible mismatch between current in-memory boolean settings vs. permanent ones.\n");
 		}
 
 		if (reason & SEPOL_COMPUTEAV_TE) {
 			printf("\t\tMissing or disabled TE allow rule.\n");
-			printf("\t\tAllow rules may exist but be disabled by boolean settings; check boolean settings.\n");
-			printf("\t\tYou can see the necessary allow rules by running audit2allow with this audit message as input.\n");
+			printf
+			    ("\t\tAllow rules may exist but be disabled by boolean settings; check boolean settings.\n");
+			printf
+			    ("\t\tYou can see the necessary allow rules by running audit2allow with this audit message as input.\n");
 		}
 
 		if (reason & SEPOL_COMPUTEAV_CONS) {
 			printf("\t\tConstraint violation.\n");
 			printf("\t\tCheck policy/constraints.\n");
-			printf("\t\tTypically, you just need to add a type attribute to the domain to satisfy the constraint.\n");
+			printf
+			    ("\t\tTypically, you just need to add a type attribute to the domain to satisfy the constraint.\n");
 		}
 
 		if (reason & SEPOL_COMPUTEAV_RBAC) {
