@@ -7,30 +7,30 @@
 #include <selinux/selinux.h>
 #include <sepol/sepol.h>
 #ifdef USE_NLS
-#include <locale.h>			    /* for setlocale() */
-#include <libintl.h>			    /* for gettext() */
+#include <locale.h>		/* for setlocale() */
+#include <libintl.h>		/* for gettext() */
 #define _(msgid) gettext (msgid)
 #else
 #define _(msgid) (msgid)
 #endif
 #ifndef PACKAGE
-#define PACKAGE "policycoreutils"   /* the name of this package lang translation */
+#define PACKAGE "policycoreutils"	/* the name of this package lang translation */
 #endif
 
-void usage(char *progname) 
+void usage(char *progname)
 {
 	fprintf(stderr, _("usage:  %s [-bq]\n"), progname);
 	exit(1);
 }
 
-int main(int argc, char **argv) 
+int main(int argc, char **argv)
 {
-	int ret, opt, quiet=0, preservebools = 1, nargs;
+	int ret, opt, quiet = 0, preservebools = 1, nargs;
 
 #ifdef USE_NLS
-	setlocale (LC_ALL, "");
-	bindtextdomain (PACKAGE, LOCALEDIR);
-	textdomain (PACKAGE);
+	setlocale(LC_ALL, "");
+	bindtextdomain(PACKAGE, LOCALEDIR);
+	textdomain(PACKAGE);
 #endif
 
 	while ((opt = getopt(argc, argv, "bq")) > 0) {
@@ -51,10 +51,14 @@ int main(int argc, char **argv)
 	if (nargs > 2)
 		usage(argv[0]);
 	if (nargs >= 1) {
-		fprintf(stderr, "%s:  Warning!  Policy file argument (%s) is no longer supported, installed policy is always loaded.  Continuing...\n", argv[0], argv[optind++]);
+		fprintf(stderr,
+			"%s:  Warning!  Policy file argument (%s) is no longer supported, installed policy is always loaded.  Continuing...\n",
+			argv[0], argv[optind++]);
 	}
 	if (nargs == 2) {
-		fprintf(stderr, "%s:  Warning!  Boolean file argument (%s) is no longer supported, installed booleans file is always used.  Continuing...\n", argv[0], argv[optind++]);
+		fprintf(stderr,
+			"%s:  Warning!  Boolean file argument (%s) is no longer supported, installed booleans file is always used.  Continuing...\n",
+			argv[0], argv[optind++]);
 	}
 
 	ret = selinux_mkload_policy(preservebools);
