@@ -30,8 +30,9 @@ int policyvers = 0;
 
 static void usage(char *program_name)
 {
-        printf("usage: %s [-V -c [version]] basemodpkg outputfile\n", program_name);
-        exit(1);
+	printf("usage: %s [-V -a -c [version]] basemodpkg outputfile\n",
+	       program_name);
+	exit(1);
 }
 
 int main(int argc, char **argv)
@@ -42,31 +43,42 @@ int main(int argc, char **argv)
 	sepol_module_package_t *base;
 	sepol_policydb_t *out, *p;
 	FILE *fp, *outfile;
+	int check_assertions = 1;
 
-	while ((ch = getopt(argc, argv, "c:Vv")) != EOF) {
-                switch (ch) {
-                case 'V':
-                        show_version = 1;
-                        break;
-                case 'v':
-                	verbose = 1;
+	while ((ch = getopt(argc, argv, "c:Vva")) != EOF) {
+		switch (ch) {
+		case 'V':
+			show_version = 1;
 			break;
-		case 'c': {
-			long int n = strtol(optarg, NULL, 10);
-			if (errno) {
-				fprintf(stderr, "%s:  Invalid policyvers specified: %s\n", argv[0], optarg);
-				usage(argv[0]);
-				exit(1);
-			}
-			if (n < sepol_policy_kern_vers_min() || n > sepol_policy_kern_vers_max()) {
-				fprintf(stderr, "%s:  policyvers value %ld not in range %d-%d\n",
-					argv[0], n, sepol_policy_kern_vers_min(), sepol_policy_kern_vers_max());
-				usage(argv[0]);
-				exit(1);
-			}
-			policyvers = n;
+		case 'v':
+			verbose = 1;
 			break;
-		}
+		case 'c':{
+				long int n = strtol(optarg, NULL, 10);
+				if (errno) {
+					fprintf(stderr,
+						"%s:  Invalid policyvers specified: %s\n",
+						argv[0], optarg);
+					usage(argv[0]);
+					exit(1);
+				}
+				if (n < sepol_policy_kern_vers_min()
+				    || n > sepol_policy_kern_vers_max()) {
+					fprintf(stderr,
+						"%s:  policyvers value %ld not in range %d-%d\n",
+						argv[0], n,
+						sepol_policy_kern_vers_min(),
+						sepol_policy_kern_vers_max());
+					usage(argv[0]);
+					exit(1);
+				}
+				policyvers = n;
+				break;
+			}
+		case 'a':{
+				check_assertions = 0;
+				break;
+			}
 		default:
 			usage(argv[0]);
 		}
@@ -74,20 +86,22 @@ int main(int argc, char **argv)
 
 	if (verbose) {
 		if (policyvers)
-			printf("Building version %d policy\n",policyvers);
+			printf("Building version %d policy\n", policyvers);
+	}
+
+	if (show_version) {
+		printf("%s\n", EXPANDPOLICY_VERSION);
+		exit(0);
+	}
+
+	/* check args */
+	if (argc < 3 || !(optind != (argc - 1))) {
+		fprintf(stderr,
+			"%s:  You must provide the base module package and output filename\n",
+			argv[0]);
+		usage(argv[0]);
 	}
 
-        if (show_version) {
-                printf("%s\n", EXPANDPOLICY_VERSION);
-                exit(0);
-        }
-        
-        /* check args */
-        if (argc < 3 || !(optind != (argc - 1))) {
-                fprintf(stderr, "%s:  You must provide the base module package and output filename\n", argv[0]);
-                usage(argv[0]);
-        }
-        
 	basename = argv[optind++];
 	outname = argv[optind];
 
@@ -95,7 +109,7 @@ int main(int argc, char **argv)
 		fprintf(stderr, "%s:  Out of memory\n", argv[0]);
 		exit(1);
 	}
-	  
+
 	/* read the base module */
 	if (sepol_module_package_create(&base)) {
 		fprintf(stderr, "%s:  Out of memory\n", argv[0]);
@@ -110,7 +124,7 @@ int main(int argc, char **argv)
 	sepol_policy_file_set_fp(pf, fp);
 	ret = sepol_module_package_read(base, pf, 0);
 	if (ret) {
-		fprintf(stderr, "%s:  Error in reading package from %s\n", 
+		fprintf(stderr, "%s:  Error in reading package from %s\n",
 			argv[0], basename);
 		exit(1);
 	}
@@ -122,44 +136,46 @@ int main(int argc, char **argv)
 		fprintf(stderr, "%s:  Error while enabling avrules\n", argv[0]);
 		exit(1);
 	}
-	
+
 	/* create the output policy */
 
-        if (sepol_policydb_create(&out)) {
-                fprintf(stderr, "%s:  Out of memory\n", argv[0]);
-                exit(1);
-        }
+	if (sepol_policydb_create(&out)) {
+		fprintf(stderr, "%s:  Out of memory\n", argv[0]);
+		exit(1);
+	}
 
-        if (sepol_expand_module(NULL, p, out, verbose, 1)) {
-        	fprintf(stderr, "%s:  Error while expanding policy\n", argv[0]);
-        	exit(1);
-        }
+	if (sepol_expand_module(NULL, p, out, verbose, check_assertions)) {
+		fprintf(stderr, "%s:  Error while expanding policy\n", argv[0]);
+		exit(1);
+	}
 
 	if (policyvers) {
 		if (sepol_policydb_set_vers(out, policyvers)) {
-			fprintf(stderr, "%s:  Invalid version %d\n", argv[0], policyvers);
+			fprintf(stderr, "%s:  Invalid version %d\n", argv[0],
+				policyvers);
 			exit(1);
 		}
 	}
-        
-        sepol_module_package_free(base);
-        
-        outfile = fopen(outname, "w");
-        if (!outfile) {
-        	perror(outname);
-        	exit(1);	
-        }
+
+	sepol_module_package_free(base);
+
+	outfile = fopen(outname, "w");
+	if (!outfile) {
+		perror(outname);
+		exit(1);
+	}
 
 	sepol_policy_file_set_fp(pf, outfile);
 	ret = sepol_policydb_write(out, pf);
 	if (ret) {
-		fprintf(stderr, "%s:  Error while writing expanded policy to %s\n",
+		fprintf(stderr,
+			"%s:  Error while writing expanded policy to %s\n",
 			argv[0], outname);
 		exit(1);
 	}
 	fclose(outfile);
 	sepol_policydb_free(out);
 	sepol_policy_file_free(pf);
-	
+
 	return 0;
 }
