@@ -18,10 +18,11 @@ int permanent = 0;
 
 int setbool(char **list, size_t start, size_t end);
 
-
 void usage(void)
 {
-	fputs("\nUsage:  setsebool [ -P ] boolean value | bool1=val1 bool2=val2...\n\n", stderr);
+	fputs
+	    ("\nUsage:  setsebool [ -P ] boolean value | bool1=val1 bool2=val2...\n\n",
+	     stderr);
 	exit(1);
 }
 
@@ -29,7 +30,7 @@ int main(int argc, char **argv)
 {
 	size_t rc, start;
 
-	if (argc < 2) 
+	if (argc < 2)
 		usage();
 
 	if (is_selinux_enabled() <= 0) {
@@ -39,11 +40,10 @@ int main(int argc, char **argv)
 
 	if (strcmp(argv[1], "-P") == 0) {
 		permanent = 1;
-		if (argc < 3) 
+		if (argc < 3)
 			usage();
 		start = 2;
-	}
-	else
+	} else
 		start = 1;
 
 	/* Check to see which way we are being called. If a '=' is passed,
@@ -57,63 +57,61 @@ int main(int argc, char **argv)
 			usage();
 
 		/* Add 1 for the '=' */
-		len = strlen(argv[start]) + strlen(argv[start+1]) + 2;
-		bool_list[0]=(char *)malloc(len);
+		len = strlen(argv[start]) + strlen(argv[start + 1]) + 2;
+		bool_list[0] = (char *)malloc(len);
 		if (bool_list[0] == 0) {
 			fputs("Out of memory - aborting\n", stderr);
 			return 1;
 		}
-		snprintf(bool_list[0], len, "%s=%s", argv[start], 
-							argv[start+1]);
+		snprintf(bool_list[0], len, "%s=%s", argv[start],
+			 argv[start + 1]);
 		rc = setbool(bool_list, 0, 1);
 		free(bool_list[0]);
-	}
-	else 
+	} else
 		rc = setbool(argv, start, argc);
 
 	return rc;
 }
 
 /* Apply boolean changes to policy via libselinux */
-static int selinux_set_boolean_list(
-	size_t boolcnt, 
-	SELboolean *boollist, 
-	int permanent) {
+static int selinux_set_boolean_list(size_t boolcnt,
+				    SELboolean * boollist, int perm)
+{
 
-	if (security_set_boolean_list(boolcnt, boollist, permanent)) {
+	if (security_set_boolean_list(boolcnt, boollist, perm)) {
 		if (errno == ENOENT)
 			fprintf(stderr, "Could not change active booleans: "
 				"Invalid boolean\n");
 		else if (errno)
 			perror("Could not change active booleans");
-	
+
 		return -1;
 	}
-	
+
 	return 0;
 }
 
 /* Apply (permanent) boolean changes to policy via libsemanage */
-static int semanage_set_boolean_list(
-	size_t boolcnt, 
-	SELboolean *boollist,
-	int permanent) {
+static int semanage_set_boolean_list(size_t boolcnt,
+				     SELboolean * boollist, int perm)
+{
 
 	size_t j;
-	semanage_handle_t* handle = NULL;
-	semanage_bool_t* boolean = NULL;
-	semanage_bool_key_t* bool_key = NULL;
+	semanage_handle_t *handle = NULL;
+	semanage_bool_t *boolean = NULL;
+	semanage_bool_key_t *bool_key = NULL;
 	int managed;
 
 	handle = semanage_handle_create();
 	if (handle == NULL) {
-		fprintf(stderr, "Could not create semanage library handle\n"); 
+		fprintf(stderr, "Could not create semanage library handle\n");
 		goto err;
 	}
 
 	managed = semanage_is_managed(handle);
 	if (managed < 0) {
-		fprintf(stderr, "Error when checking whether policy is managed\n"); 
+		fprintf(stderr,
+			"Error when checking whether policy is managed\n");
 		goto err;
 
 	} else if (managed == 0) {
@@ -129,11 +127,12 @@ static int semanage_set_boolean_list(
 		goto err;
 
 	for (j = 0; j < boolcnt; j++) {
-		
-		if (semanage_bool_create(handle, &boolean) < 0) 
+
+		if (semanage_bool_create(handle, &boolean) < 0)
 			goto err;
 
-		if (semanage_bool_set_name(handle, boolean, boollist[j].name) < 0)
+		if (semanage_bool_set_name(handle, boolean, boollist[j].name) <
+		    0)
 			goto err;
 
 		semanage_bool_set_value(boolean, boollist[j].value);
@@ -141,18 +140,21 @@ static int semanage_set_boolean_list(
 		if (semanage_bool_key_extract(handle, boolean, &bool_key) < 0)
 			goto err;
 
-		if (permanent && semanage_bool_modify_local(handle, bool_key, boolean) < 0)
+		if (perm
+		    && semanage_bool_modify_local(handle, bool_key,
+						  boolean) < 0)
 			goto err;
 
 		if (semanage_bool_set_active(handle, bool_key, boolean) < 0) {
-			fprintf(stderr, "Could not change boolean %s\n", boollist[j].name);
+			fprintf(stderr, "Could not change boolean %s\n",
+				boollist[j].name);
 			goto err;
 		}
 		semanage_bool_key_free(bool_key);
 		semanage_bool_free(boolean);
 		bool_key = NULL;
 		boolean = NULL;
-	}	
+	}
 
 	semanage_set_reload(handle, 0);
 	if (semanage_commit(handle) < 0)
@@ -160,11 +162,11 @@ static int semanage_set_boolean_list(
 
 	semanage_disconnect(handle);
 
-        out:
+      out:
 	semanage_handle_destroy(handle);
 	return 0;
 
-	err:
+      err:
 	semanage_bool_key_free(bool_key);
 	semanage_bool_free(boolean);
 	semanage_handle_destroy(handle);
@@ -177,11 +179,11 @@ static int semanage_set_boolean_list(
 int setbool(char **list, size_t start, size_t end)
 {
 	char *name, *value_ptr;
-	int j=0, value;
+	int j = 0, value;
 	size_t i = start;
-	size_t boolcnt=end-start;
+	size_t boolcnt = end - start;
 	struct passwd *pwd;
-	SELboolean *vallist=calloc(boolcnt, sizeof(SELboolean));
+	SELboolean *vallist = calloc(boolcnt, sizeof(SELboolean));
 	if (!vallist)
 		goto omem;
 
@@ -189,23 +191,24 @@ int setbool(char **list, size_t start, size_t end)
 		name = list[i];
 		value_ptr = strchr(list[i], '=');
 		if (value_ptr == 0) {
-			fprintf(stderr, 
-			"setsebool: '=' not found in boolean expression %s\n",
+			fprintf(stderr,
+				"setsebool: '=' not found in boolean expression %s\n",
 				list[i]);
 			goto err;
 		}
 		*value_ptr = 0;
 		value_ptr++;
-		if (strcmp(value_ptr, "1") == 0 || 
-				strcasecmp(value_ptr, "true") == 0)
+		if (strcmp(value_ptr, "1") == 0 ||
+		    strcasecmp(value_ptr, "true") == 0 ||
+		    strcasecmp(value_ptr, "on") == 0)
 			value = 1;
-		else if (strcmp(value_ptr, "0") == 0 || 
-				strcasecmp(value_ptr, "false") == 0)
+		else if (strcmp(value_ptr, "0") == 0 ||
+			 strcasecmp(value_ptr, "false") == 0 ||
+			 strcasecmp(value_ptr, "off") == 0)
 			value = 0;
 		else {
 			fprintf(stderr, "setsebool: illegal value "
-					"%s for boolean %s\n",
-					value_ptr, name);
+				"%s for boolean %s\n", value_ptr, name);
 			goto err;
 		}
 
@@ -219,7 +222,7 @@ int setbool(char **list, size_t start, size_t end)
 		/* Now put it back */
 		value_ptr--;
 		*value_ptr = '=';
-	} 
+	}
 
 	if (semanage_set_boolean_list(boolcnt, vallist, permanent) < 0)
 		goto err;
@@ -234,27 +237,28 @@ int setbool(char **list, size_t start, size_t end)
 		value_ptr++;
 		if (pwd && pwd->pw_name)
 			syslog(LOG_NOTICE,
-				"The %s policy boolean was changed to %s by %s",
-				name, value_ptr, pwd->pw_name);
+			       "The %s policy boolean was changed to %s by %s",
+			       name, value_ptr, pwd->pw_name);
 		else
 			syslog(LOG_NOTICE,
-				"The %s policy boolean was changed to %s by uid:%d",
-				name, value_ptr, getuid());
+			       "The %s policy boolean was changed to %s by uid:%d",
+			       name, value_ptr, getuid());
 		i++;
 	}
 
-	for (i=0; i < boolcnt; i++)
+	for (i = 0; i < boolcnt; i++)
 		free(vallist[i].name);
 	free(vallist);
 	return 0;
-		
-	omem:
+
+      omem:
 	fprintf(stderr, "setsebool: out of memory");
-		
-	err:
-	for (i=0; i < boolcnt; i++) 
-		free(vallist[i].name);
-	free(vallist);
+
+      err:
+	if (vallist) {
+		for (i = 0; i < boolcnt; i++)
+			free(vallist[i].name);
+		free(vallist);
+	}
 	return -1;
 }
-
